#ifndef __DEFS_H__
#define __DEFS_H__
/*
 * Copyright (c) 1983 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * $Id: defs.h,v 1.6 2001/03/12 18:16:30 kim Exp $
 * @(#)defs.h      5.2 (Berkeley) 3/20/86
 */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <pwd.h>
#include <grp.h>
#include <syslog.h>
#include <setjmp.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/stat.h>

#ifndef __P
# ifdef __STDC__
#  define __P(a)	a
# else 
#  define __P(a)	()
# endif
#endif

#ifndef __GNUC__
# ifndef __attribute__
#  define __attribute__(a)
# endif
#endif

#ifndef __STDC__
# ifndef const
#  define const
# endif
#endif

#include "version.h"
#include "config-def.h"
#include "config.h"
#include "config-data.h"
#include "pathnames.h"
#include "types.h"
#include "filesys.h"

#if	!defined(yacc)
#include "y.tab.h"
#endif	/* yacc */

#include <signal.h>

/*
 * This belongs in os-svr4.h but many SVR4 OS's
 * define SVR4 externel to Rdist so we put this
 * check here.
 */
#if	defined(SVR4)
#define NEED_FCNTL_H
#define NEED_UNISTD_H
#define NEED_NETDB_H
#endif	/* defined(SVR4) */

#if	defined(NEED_NETDB_H)
#include <netdb.h>
#endif	/* NEED_NETDB_H */
#if	defined(NEED_FCNTL_H)
#include <fcntl.h>
#endif	/* NEED_FCNTL_H */
#if	defined(NEED_LIMITS_H)
#include <limits.h>
#endif	/* NEED_LIMITS_H */
#if	defined(NEED_UNISTD_H)
#include <unistd.h>
#endif	/* NEED_UNISTD_H */
#if	defined(NEED_STRING_H)
#include <string.h>
#endif	/* NEED_STRING_H */

#if defined(ARG_TYPE)
#if	ARG_TYPE == ARG_STDARG
#include <stdarg.h>
#endif
#if	ARG_TYPE == ARG_VARARGS
#include <varargs.h>
#endif
#endif	/* ARG_TYPE */

	/* boolean truth */
#ifndef TRUE
#define TRUE		1
#endif
#ifndef FALSE
#define FALSE		0
#endif

	/* file modes */
#ifndef S_IXUSR
#define S_IXUSR		0000100
#endif
#ifndef S_IXGRP
#define S_IXGRP		0000010
#endif
#ifndef S_IXOTH
#define S_IXOTH		0000001
#endif

	/* lexical definitions */
#define	QUOTECHAR	160	/* quote next character */

	/* table sizes */
#define HASHSIZE	1021
#define INMAX		3500

	/* expand type definitions */
#define E_VARS		0x1
#define E_SHELL		0x2
#define E_TILDE		0x4
#define E_ALL		0x7

	/* actions for lookup() */
#define LOOKUP		0
#define INSERT		1
#define REPLACE		2

	/* Bit flag test macros */
#define IS_ON(b,f)	(b > 0 && (b & f))
#define IS_OFF(b,f)	!(IS_ON(b,f))
#define FLAG_ON(b,f)	b |= f
#define FLAG_OFF(b,f)	b &= ~(f)

/*
 * POSIX systems should already have S_* defined.
 */
#ifndef S_ISDIR
#define S_ISDIR(m) 	(((m) & S_IFMT) == S_IFDIR)
#endif
#ifndef S_ISREG
#define S_ISREG(m) 	(((m) & S_IFMT) == S_IFREG)
#endif
#ifndef S_ISLNK
#define S_ISLNK(m) 	(((m) & S_IFMT) == S_IFLNK)
#endif

#define ALLOC(x) 	(struct x *) xmalloc(sizeof(struct x))
#define A(s)		((s) ? s : "<null>")

/*
 * Environment variable names
 */
#define E_FILES		"FILES"			/* List of files */
#define E_LOCFILE	"FILE"			/* Local Filename  */
#define E_REMFILE	"REMFILE"		/* Remote Filename */
#define E_BASEFILE	"BASEFILE"		/* basename of Remote File */

/*
 * Suffix to use when saving files
 */
#ifndef SAVE_SUFFIX
#define SAVE_SUFFIX	".OLD"
#endif

/*
 * Get system error string
 */
#define SYSERR 		strerror(errno)

#define COMMENT_CHAR	'#'		/* Config file comment char */
#define CNULL		'\0'		/* NULL character */

/*
 * These are the top level protocol commands.
 */
#define C_NONE		'='		/* No command - pass cleanly */
#define C_ERRMSG	'\1'		/* Log an error message */
#define C_FERRMSG	'\2'		/* Log a fatal error message */
#define C_NOTEMSG	'\3'		/* Log a note message */
#define C_LOGMSG	'\4'		/* Log a message */
#define C_ACK		'\5'		/* Acknowledge */
#define C_SETCONFIG	'c'		/* Set configuration parameters */
#define C_DIRTARGET	'T'		/* Set target directory name */
#define C_TARGET	't'		/* Set target file name */
#define C_RECVREG	'R'		/* Receive a regular file */
#define C_RECVDIR	'D'		/* Receive a directory */
#define C_RECVSYMLINK	'K'		/* Receive a symbolic link */
#define C_RECVHARDLINK	'k'		/* Receive a hard link */
#define C_END		'E'		/* Indicate end of recieve/send */
#define C_CLEAN		'C'		/* Clean up */
#define C_QUERY		'Q'		/* Query without checking */
#define C_SPECIAL	'S'		/* Execute special command */
#define C_CMDSPECIAL	's'		/* Execute cmd special command */
#define C_CHMOG		'M'		/* Chown,Chgrp,Chmod a file */

#define	ack() 		(void) sendcmd(C_ACK, (char *)NULL)
#define	err() 		(void) sendcmd(C_ERRMSG, (char *)NULL)

/*
 * Session startup commands.
 */
#define S_VERSION	'V'		/* Version number */
#define S_REMOTEUSER	'R'		/* Remote user name */
#define S_LOCALUSER	'L'		/* Local user name */
#define S_END		'E'		/* End of session startup commands */

/*
 * These are the commands for "set config".
 */
#define SC_FREESPACE	's'		/* Set min free space */
#define SC_FREEFILES	'f'		/* Set min free files */
#define SC_HOSTNAME	'H'		/* Set client hostname */
#define SC_LOGGING	'L'		/* Set logging options */
#define SC_DEFOWNER	'o'		/* Set default owner */
#define SC_DEFGROUP	'g'		/* Set default group */

/*
 * Query commands
 */
#define QC_ONNFS	'F'		/* File exists & is on a NFS */
#define QC_ONRO		'O'		/* File exists & is on a readonly fs */
#define QC_NO		'N'		/* File does not exist */
#define QC_SYM		'l'		/* File exists & is a symlink */
#define QC_YES		'Y'		/* File does exist */

/*
 * Clean commands
 */
#define CC_QUERY	'Q'		/* Query if file should be rm'ed */
#define CC_END		'E'		/* End of cleaning */
#define CC_YES		'Y'		/* File doesn't exist - remove */
#define CC_NO		'N'		/* File does exist - don't remove */

/*
 * Run Command commands
 */
#define RC_FILE		'F'		/* Name of a target file */
#define RC_COMMAND	'C'		/* Command to run */

/*
 * Name list
 */
struct namelist {		/* for making lists of strings */
	char	*n_name;
	struct	namelist *n_next;
};

/*
 * Sub command structure
 */
struct subcmd {
	short	sc_type;	/* type - INSTALL,NOTIFY,EXCEPT,SPECIAL */
	opt_t	sc_options;
	char	*sc_name;
	struct	namelist *sc_args;
	struct	subcmd *sc_next;
};

/*
 * Cmd flags
 */
#define CMD_ASSIGNED	0x01	/* This entry has been assigned */
#define CMD_CONNFAILED	0x02	/* Connection failed */
#define CMD_NOCHKNFS	0x04	/* Disable NFS checks */

/*
 * General command structure
 */
struct cmd {
	int	c_type;		/* type - ARROW,DCOLON */
	int	c_flags;	/* flags - CMD_USED,CMD_FAILED */
	char	*c_name;	/* hostname or time stamp file name */
	char	*c_label;	/* label for partial update */
	struct	namelist *c_files;
	struct	subcmd *c_cmds;
	struct	cmd *c_next;
};

/*
 * Hard link buffer information
 */
struct linkbuf {
	ino_t	inum;
	dev_t	devnum;
	int	count;
	char	*pathname;
	char	*src;
	char	*target;
	struct	linkbuf *nextp;
};

extern char	       *optarg;		/* Option argument */
extern char	       *path_remsh;	/* Remote shell command */
extern char 		host[];		/* Host name of master copy */
extern char 	       *currenthost;	/* Name of current host */
extern char 	       *progname;	/* Name of this program */
extern char 	      **realargv;	/* Real argv */
extern int		optind;		/* Option index into argv */
extern int 		contimedout;	/* Connection timed out */
extern int 		debug;		/* Debugging flag */
extern opt_t 		defoptions;	/* Default install options */
extern int 		do_fork;	/* Should we do fork()'ing */
#ifndef __STDC__
extern int 		errno;		/* System error number */
#endif
extern int 		isserver;	/* Acting as remote server */
extern int 		nerrs;		/* Number of errors seen */
extern int 		nflag;		/* NOP flag, don't execute commands */
extern opt_t 		options;	/* Global options */
extern int 		proto_version;	/* Protocol version number */
extern int 		realargc;	/* Real argc */
extern int		rem_r;		/* Remote file descriptor, reading */
extern int 		rem_w;		/* Remote file descriptor, writing */
extern void 	       *rem_rz;		/* Remote file descriptor, compress */
extern void 	       *rem_wz;		/* Remote file descriptor, compress */
extern int 		rtimeout;	/* Response time out in seconds */
extern int		setjmp_ok;	/* setjmp/longjmp flag */
extern UID_T 		userid;		/* User ID of rdist user */
extern jmp_buf 		finish_jmpbuf;	/* Setjmp buffer for finish() */
extern struct group    *gr;	/* pointer to static area used by getgrent */
extern struct linkbuf  *ihead;	/* list of files with more than one link */
extern struct passwd   *pw;	/* pointer to static area used by getpwent */
extern char defowner[64];		/* Default owner */
extern char defgroup[64];		/* Default group */
#ifdef USE_STATDB
extern int 		dostatdb;
extern int 		juststatdb;
#endif /* USE_STATDB */

/*
 * Our own declarations.
 */

/* child.c */
void waitup __P((void));
int spawn __P((struct cmd *, struct cmd *));
int setnonblocking __P((int, int));
int setnonblocking __P((int, int));

/* client.c */
char *remfilename __P((char *, char *, char *, char *, int));
int inlist __P((struct namelist *, char *));
void runcmdspecial __P((struct cmd *, opt_t));
int checkfilename __P((char *));
void freelinkinfo __P((struct linkbuf *));
void cleanup __P((int));
int install __P((char *, char *, int, int , opt_t));

/* common.c */
WRITE_RETURN_T xwrite __P((int, void *, WRITE_AMT_T));
int init __P((int, char **, char **));
void finish __P((void));
void lostconn __P((void));
void coredump __P((void));
void sighandler __P((int));
int sendcmd __P((char, char *, ...));
int remline __P((u_char *, int, int));
int readrem __P((char *, int));
int readremz __P((char *, int));
char *getusername __P((UID_T, char *, opt_t));
char *getgroupname __P((GID_T, char *, opt_t));
int response __P((void));
char *exptilde __P((char *, char *));
int becomeuser __P((void));
int becomeroot __P((void));
int setfiletime __P((char *, time_t, time_t));
char *getversion __P((void));
void runcommand __P((char *));
char *xmalloc __P((int));
char *xrealloc __P((char *, unsigned int));
char *xcalloc __P((unsigned, unsigned));
char *xstrdup __P((char *));
char *xbasename __P((char *));
char *searchpath __P((char *));
int mysetlinebuf __P((FILE *));
int getsocketpair __P((int, int, int, int[]));

/* distopt.c */
DISTOPTINFO *getdistopt __P((char *, int *));
int parsedistopts __P((char *, opt_t *, int));
char *getdistoptlist __P((void));
char *getondistoptlist __P((opt_t));

/* docmd.c */
void markassigned __P((struct cmd *, struct cmd *));
int okname __P((char *));
int except __P((char *));
void docmds __P((struct namelist *, int, char **));

/* expand.c */
struct namelist *expand __P((struct namelist *, int));
u_char *xstrchr __P((u_char *, int));
void expstr __P((u_char *));
void expsh __P((u_char *));
void matchdir __P((char *));
int execbrc __P((u_char *, u_char *));
int match __P((char *, char *));
int amatch __P((char *, u_char *));

/* filesys-os.c */
FILE *setmountent __P((const char *, const char *));
mntent_t *getmountent __P((FILE *));
void endmountent __P((FILE *));

/* filesys.c */
char *find_file __P((char *, struct stat *, int *));
mntent_t *findmnt __P((struct stat *, struct mntinfo *));
int isdupmnt __P((mntent_t *, struct mntinfo *));
void wakeup __P((int));
struct mntinfo *makemntinfo __P((struct mntinfo *));
mntent_t *getmntpt __P((char *, struct stat *, int *));
int is_nfs_mounted __P((char *, struct stat *, int *));
int is_ro_mounted __P((char *, struct stat *, int *));
int is_symlinked __P((char *, struct stat *, int *));
int getfilesysinfo __P((char *, long *, long *));

/* gram.c */
int yylex __P((void));
int any __P((int, char *));
void insert __P((char *, struct namelist *, struct namelist *, struct subcmd *));
void append __P((char *, struct namelist *, char *, struct subcmd *));
void yyerror __P((char *));
struct namelist *makenl __P((char *));
struct subcmd *makesubcmd __P((int));
int yyparse __P((void));

/* hasmntopt.c */
#ifdef NEED_HASMNTOPT
char *hasmntopt __P((struct mntent *, char *));
#endif

/* isexec.c */
int isexec __P((char *, struct stat *));

/* lookup.c */
void define __P((char *));
struct namelist *lookup __P((char *, int, struct namelist *));

/* message.c */
void msgprusage __P((void));
void msgprconfig __P((void));
char *msgparseopts __P((char *, int));
void checkhostname __P((void));
void message __P((int, char *, ...));
void debugmsg __P((int, char *, ...));
void error __P((char *, ...));
void fatalerr __P((char *, ...));
char *getnotifyfile __P((void));

/* rdist.c */
FILE *opendist __P((char *));
void docmdargs __P((int, char *[]));
char *getnlstr __P((struct namelist *));

/* regex.c */
#if defined(NEED_REGEX) || !defined(NEED_RE_COMP_H)
char *re_comp __P((char *));
int re_exec __P((char *));
int backref __P((int, char *));
int cclass __P((char *, int , int));
#else
#include <re_comp.h>
#endif

/* rshrcmd.c */
int rshrcmd __P((char **, u_short, char *, char *, char *, int *));

/* server.c */
void server __P((void));

/* setargs.c */
void setargs_settup __P((int, char **, char **));
void _setproctitle __P((char *));
void setproctitle __P((const char *, ...));

/* setprogname.c */
#ifdef NEED_SETPROGNAME
void setprogname __P((const char *));
#endif

/* signal.c */
#ifdef NEED_SIGBLOCK
int sigblock __P((int));
#endif
#ifdef NEED_SIGMASK
int sigsetmask __P((int));
#endif

/* strcasecmp.c */
#ifdef NEED_STRCASECMP
int strcasecmp __P((char *, char *));
int strncasecmp __P((char *, char *, int));
#endif

/* strerror.c */
#ifdef NEED_STRERROR
char *strerror __P((int));
#endif

/* strtol.c */
#ifdef NEED_STRTOL
long strtol __P((char *, char **, int));
#endif

/* unvis.c */
#ifdef NEED_VIS
int unvis __P((char *, int, int *, int ));
int strunvis __P((char *, const char *));
#endif

/* vis.c */
#ifdef NEED_VIS
char *vis __P((char *, int, int, int ));
int strvis __P((char *, const char *, int));
int strvisx __P((char *, const char *, size_t, int));
#endif

/* vsnprintf.c */
#ifdef NEED_VSNPRINTF
int vsnprintf __P((char *, size_t, const char *, va_list));
int snprintf __P((char *, size_t, const char *, ...));
#endif

/* zopen.c */
int zwrite __P((void *, const char *, int));
int zclose __P((void *, int *));
int zread __P((void *, char *, int));
void *zopenfd __P((int, char *, int));

#include <vis.h>
#define DECODE(a, b)	strunvis(a, b)
#define ENCODE(a, b)	strvis(a, b, VIS_WHITE)

#endif	/* __DEFS_H__ */
